# -*- coding: utf-8 -*-
"""
Created on Thu Aug 22 11:12:10 2019

scripts that reads the time and locations of SPI, Multicore and Boxcore deployment per cruise.
it also converts the lat/lon coordinates to ETRS89 UTM31N x,y pairs.
results are stored in csv files with format:
    yyyy_mm_'LOCATION'_'LOCCODE'_bedsampling.csv

$Id: casino2csv_bedsampling.py 46 2020-10-22 07:25:29Z erikhendriks $
$Date: 2020-10-22 09:25:29 +0200 (do, 22 okt 2020) $
$Author: erikhendriks $
$Revision: 46 $
$HeadURL: https://svn.citg.tudelft.nl/sandbox/trunk/cruises/scripts/casino2csv_bedsampling.py $

@author: 
    erikhendriks
"""

#import packages

from pathlib import Path
import pandas as pd
from pyproj import Proj, transform
import os
import sys
from datetime import datetime as dt
import numpy as np
import re
from shapely.geometry import Point, Polygon
import geopandas as gpd

# device names
lst_PEL = ['MC','SPI','BOX']
pat_PEL = '|'.join(lst_PEL)
lst_NAV = ['MISC','ADCP','BOX']
pat_NAV = '|'.join(lst_NAV)
inProj = Proj(init='epsg:4326')
outProj = Proj(init='epsg:25831')

# define directories
scriptsDir = Path(os.path.abspath(sys.argv[0]))
baseDir = scriptsDir.parents[2] #for standard layout of repository, this gives \\trunk directory
rawDir = baseDir / 'cruises' / 'raw' 
campaignDirs = [x for x in rawDir.glob('*/')] # find all directories in this directory
campaignCode=[str(x).split(sep=os.sep)[-1] for x in campaignDirs] # this includes: year_month_location_code
outputDir=baseDir / 'cruises' / 'meta' 
ldb = gpd.read_file(str(outputDir / 'ldb_line.shp'))

# coordinate conversion from DMS to DD
def NAVstr_to_dec(coordstr):
    d=float(re.sub("[^0-9]","",coordstr.split()[-3])) #degrees
    m=float(re.sub("[^0-9]","",coordstr.split()[-2]))  #minutes
    s=float(re.sub("[^.0-9]","",coordstr.split()[-1])) #milliseconds
    DD= d +  m / 60 + s / 3600
    return str(DD)



for idx,campaignDir in enumerate(campaignDirs):
    
    # load CASINO file
    clist=[x for x in campaignDir.glob('CASINO*.csv')]
    if '2019_05_egmond_sbvn' not in str(campaignDir):
        casino=pd.read_csv(clist[0],delimiter='\t',encoding = "ISO-8859-1",low_memory=False)
    else:
        casino=pd.read_csv(clist[0],delimiter=',',encoding = "ISO-8859-1")
        
        # get the 'operations' from casino
    operations=casino.loc[casino['Type']=='OPE',:]
    operations.loc[:,'DEVICE CODE']= operations.loc[:,'Device code'].str.upper()
    #select the operations where the device that was deployed matches the device list as indicated before
    if '2019_05_egmond_sbvn' not in str(campaignDir): #PELAGIA
        deploy=operations[operations['DEVICE CODE'].str.contains(pat_PEL)]
        deploy.loc[:,'DEVICE CODE']=deploy.replace({'DEVICE CODE' : { 'MC12' : 'MC', 'BOX125' : 'BOX'}})  # replace strings if needed
    else: #NAVICULA
        deploy=operations[operations['DEVICE CODE'].str.contains(pat_NAV)]
        deploy.loc[:,'DEVICE CODE']=deploy.replace({'DEVICE CODE' : { 'MISC' : 'MC','ADCP' : 'MC', 'BOXC' : 'BOX'}})  # replace strings if needed
        
    deploy.loc[:,'datetime']=pd.to_datetime(deploy['Date'] +' '+ deploy['Heure'],utc=True, format=' %d/%m/%Y %H:%M:%S')
   
    
    
    # create dataframe that will be output to csv by selecting small selection of columns
    # then save the dataframe to a csv file
    


    if '2019_05_egmond_sbvn' not in str(campaignDir):
        output=deploy.loc[:,['Latitude (degres, minutes)','Longitude (degres, minutes)','Device name','Device code',
                         'Operation Id','Observation','datetime','DEVICE CODE']].copy()
        #rename lat/lon columns
        output.rename(columns={'Latitude (degres, minutes)':'lat','Longitude (degres, minutes)':'lon'},inplace=True)
        # transform the lat, lon pair to x,y coordinates
        output.loc[:,'ETRS89_UTM31N_x'],output.loc[:,'ETRS89_UTM31N_y'] = transform(inProj,outProj,output.loc[:,'lon'].values,output.loc[:,'lat'].values) 
        
       
    else:
        output=deploy.loc[:,['Latitude(deg. min.milli)','Longitude(deg. min.milli)','Device name','Device code',
                         'Operation Id','Observation','datetime','DEVICE CODE']].copy()
        output.rename(columns={'Latitude(deg. min.milli)':'lat','Longitude(deg. min.milli)':'lon'},inplace=True)
        output['lat_DD']=output.apply(lambda row: NAVstr_to_dec(*row[['lat']]),axis=1)
        output['lon_DD']=output.apply(lambda row: NAVstr_to_dec(*row[['lon']]),axis=1)
        output.loc[:,'ETRS89_UTM31N_x'],output.loc[:,'ETRS89_UTM31N_y'] = transform(inProj,outProj,output.loc[:,'lon_DD'].values,output.loc[:,'lat_DD'].values) 
    
    #make datetime the index column
    output.index=output.loc[:,'datetime']
    output.index.name = 'datetime'
    output.drop(labels='datetime',axis=1)

    # for the Egmond campaigns, use coordinates to find the station numbers
# for each entry in geodataframe, find distance to landboundary
    if 'sbns' in str(campaignDir): 
        gdf = gpd.GeoDataFrame(output, 
                               geometry=[Point(x, y) for x, y in zip(output.ETRS89_UTM31N_x, output.ETRS89_UTM31N_y)],
                               crs='epsg:25831')
        gdf['distan']=0
        for idxx, row in gdf.iterrows():
            gdf.loc[idxx,'distan']=row.geometry.distance(ldb.loc[0,'geometry'])
        gdf['station']=np.round(gdf['distan']/500)-1 # find station corresponding with cross-shore distance
        gdf['cumul']=gdf.groupby(['station','DEVICE CODE']).cumcount()
        output[['distan','station','cumul']]=gdf.loc[:,['distan','station','cumul']]
#if write_f
#    gdf.to_csv(outputDir / 'spi_overview.csv')
        
    

    #... and save to file    
    output.to_csv(outputDir / str(campaignCode[idx]+'_bedsampling.csv'))
    #    'ETRS89_UTM31N_x','ETRS89_UTM31N_y'
#        for frame in deploy['DEVICE CODE'].unique():
#            output.loc[frame,'lat']=deploy.loc[(deploy['Action name'].str.lower()=='deployment') & (deploy['DEVICE CODE']==frame),'Latitude (degres, minutes)'].values[0]
#            output.loc[frame,'lon']=deploy.loc[(deploy['Action name'].str.lower()=='deployment') & (deploy['DEVICE CODE']==frame),'Longitude (degres, minutes)'].values[0]
#            output.loc[frame,'date_begin']=deploy.loc[(deploy['Action name'].str.lower()=='deployment') & (deploy['DEVICE CODE']==frame),'datetime'].values[0]
#            output.loc[frame,'date_end']=deploy.loc[(deploy['Action name'].str.lower()=='recovery') & (deploy['DEVICE CODE']==frame),'datetime'].values[0]
#    else:
#        output=pd.DataFrame(index=deploy['DEVICE CODE'].unique(),
#                            columns=['lat','lon','ETRS89_UTM31N_x','ETRS89_UTM31N_y','date_begin','date_end','date_begin2','date_end2'])
#        output.index.name = 'frame'
#        for frame in deploy['DEVICE CODE'].unique():
#            output.loc[frame,'lat']=deploy.loc[(deploy['Action name'].str.lower()=='deployment') & (deploy['DEVICE CODE']==frame),'Latitude (degres, minutes)'].values[0]
#            output.loc[frame,'lon']=deploy.loc[(deploy['Action name'].str.lower()=='deployment') & (deploy['DEVICE CODE']==frame),'Longitude (degres, minutes)'].values[0]
#            output.loc[frame,'date_begin']=deploy.loc[(deploy['Action name'].str.lower()=='deployment') & (deploy['DEVICE CODE']==frame),'datetime'].values[0]
#            output.loc[frame,'date_end']=deploy.loc[(deploy['Action name'].str.lower()=='recovery') & (deploy['DEVICE CODE']==frame),'datetime'].values[0]        
#            if frame=='EGM':
#                output.loc[frame,'date_begin2']=deploy.loc[(deploy['Action name'].str.lower()=='deployment') & (deploy['DEVICE CODE']==frame),'datetime'].values[1]
#                output.loc[frame,'date_end2']=deploy.loc[(deploy['Action name'].str.lower()=='recovery') & (deploy['DEVICE CODE']==frame),'datetime'].values[1]      
#                output.loc[:,'date_begin2']=pd.to_datetime(output['date_begin2'],utc=True) 
#                output.loc[:,'date_end2']=pd.to_datetime(output['date_end2'],utc=True) 
#    # convert date_begin and date_end to datetime64[ns] format
#    output.loc[:,'date_begin']=pd.to_datetime(output['date_begin'],utc=True) 
#    output.loc[:,'date_end']=pd.to_datetime(output['date_end'],utc=True) 
#   
      
    
#   





